var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { IAMClient, DeleteUserCommand, DeleteLoginProfileCommand, IAMServiceException, DeleteUserPolicyCommand, DetachUserPolicyCommand, RemoveUserFromGroupCommand, } from '@aws-sdk/client-iam';
import { deleteAccessKey } from './deleteAccessKey';
import { listAccessKeys } from './listAccessKeys';
import { listUserGroups } from './listUserGroups';
import { listUserInlinePolicies } from './listUserInlinePolicies';
import { listUserPolicies } from './listUserPolicies';
/**
 * Deletes an IAM user. The full logic involved when deleting a user is discussed here:
 *
 * https://docs.aws.amazon.com/AWSJavaScriptSDK/v3/latest/clients/client-iam/classes/deleteusercommand.html
 * https://docs.aws.amazon.com/IAM/latest/UserGuide/id_users_manage.html#id_users_deleting
 * https://docs.aws.amazon.com/IAM/latest/APIReference/API_DeleteUser.html
 *
 * For this application, we only address the following:
 *
 * - login profile
 * - access keys
 * - inline policies
 * - attached policies
 * - group memberships
 *
 */
export function deleteUser({ accessKey, secretKey, region, userName }) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new IAMClient({
            region,
            credentials: {
                accessKeyId: accessKey,
                secretAccessKey: secretKey,
            },
        });
        // Delete login profile
        try {
            yield client.send(new DeleteLoginProfileCommand({ UserName: userName }));
        }
        catch (error) {
            if (error instanceof IAMServiceException && error.name === 'NoSuchEntity') {
                // expected if there was no login profile, so this is normal, no need to throw an exception
            }
            else {
                throw error;
            }
        }
        // Delete inline policies
        const policyNames = yield listUserInlinePolicies({
            accessKey,
            secretKey,
            region,
            userName,
        });
        for (const policyName of policyNames) {
            yield client.send(new DeleteUserPolicyCommand({
                UserName: userName,
                PolicyName: policyName,
            }));
        }
        // Detach managed policies
        const policies = yield listUserPolicies({
            accessKey,
            secretKey,
            region,
            userName,
        });
        for (const policy of policies) {
            yield client.send(new DetachUserPolicyCommand({
                UserName: userName,
                PolicyArn: policy.PolicyArn,
            }));
        }
        // Remove from groups
        const groups = yield listUserGroups({
            accessKey,
            secretKey,
            region,
            userName,
        });
        for (const group of groups) {
            yield client.send(new RemoveUserFromGroupCommand({
                UserName: userName,
                GroupName: group.GroupName,
            }));
        }
        // Delete access keys
        const accessKeys = yield listAccessKeys({
            accessKey,
            secretKey,
            region,
            userName,
        });
        for (const userAccessKey of accessKeys) {
            yield deleteAccessKey({
                accessKey,
                secretKey,
                region,
                userName,
                accessKeyId: userAccessKey.AccessKeyId,
            });
        }
        // Finally, we are ready to delete the user
        yield client.send(new DeleteUserCommand({ UserName: userName }));
    });
}
//# sourceMappingURL=deleteUser.js.map